//-----------------------------------------------------------------------------
//
// Program Description:
// --------------------
//
// This code takes and averages 10 digital measurements from input
// P1.4 (collector of optical sensor). It then uses that information to
// calculate and print, to a terminal window via the UART, angular speed
// (rev/s and RPM) and linear speed (ft/s and MPH) of the radius of the
// spinning disc. Also, the measured data is sent to an 8-digit LCD screen.
// And because the LCD is limited to 8-digits, not all the measured data can
// be displayed simultaneously. Therefore, a keypad is used to select the
// desired measured data (rev/s, RPM, ft/s, MPH).
//
// The system is clocked by the internal 24.5MHz oscillator.  Timer 0 triggers
// on its overflow every 100us, which, in turn, generates a 100us timer.
//
// When the averaging of 10 measurements is complete the value is printed to the
// terminal via printf before starting another average cycle.
//
// Terminal output is done via printf, which directs the characters to
// UART0.  A UART initialization routine is therefore necessary.
//
//
// F93x Resources:
// ---------------
// Timer0: overflow initiates 100us timer tick.
// Timer1: clocks UART
//
// How To Test:
// ------------
// 1) Ensure that jumpers are placed on the following:
//       J12:  P0.4/TX
//             VDD/VIO
//       J11:  PWR/WALL_PWR
//       J17:  VDD_PIN/PWR
// 2) Connect the hardware as shown in the associated schematics/connectivity
//	  diagram.
// 3) Connect a USB cable from the development board P3 to a PC.
// 4) On the PC, open HyperTerminal (or any other terminal program) and connect
//    to the USB port (virtual com port) at <230400> and 8-N-1.
// 5) Connect an optical sensor's open-collector to P1.4.
// 6) Connect the USB Debug Adapter to J9.
// 7) Turn the POWER switch (SW5) to the "ON" position.
// 8) Compile and download code to a 'F93x device on a C8051F93x-TB development
//    board by selecting Run -> Debug from the menus, clicking the Debug button
//    in the quick menu, or pressing F11.
// 9) Run the code by selecting Run -> Resume from the menus, clicking the
//    Resume button in the quick menu, or pressing F8.
// 10) A 5-inch diameter disc should be clear/transparent except for one small
//	  line (2-3mm in width) opaque line. This opaque line is what will trigger
//	  the optical sensor.
// 11) HyperTerminal will print the data if everything is working properly.
// 12) The CP2400-GQ LCD dev board will display the measured data.
//
// Target:         C8051F92x-C8051F93x

//-----------------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------------
#include <compiler_defs.h>
#include <SI_C8051F930_Register_Enums.h>	// SFR declarations
#include "InitDevice.h"
#include <stdio.h>
#include "..\LCD_LIB\lcd_lib.h"

//-----------------------------------------------------------------------------
// Pin Declarations
//-----------------------------------------------------------------------------
SBIT (OpticalSensor, SFR_P1, 4);	// Declare variable: 1 = Optical Sensor blocked; 0 = Optical Sensor clear.
SBIT (YELLOW_LED, SFR_P1, 6);		// LED=1 means ON

SBIT (Row_1, SFR_P2, 0);			//
SBIT (Row_2, SFR_P2, 1);			//
SBIT (Row_3, SFR_P2, 2);			//
SBIT (Row_4, SFR_P2, 3);			//

//-----------------------------------------------------------------------------
// Global CONSTANTS
//-----------------------------------------------------------------------------
char OpticalSensorFallingEdge = 0;	// Declare variable and initialize to 0.
char OpticalSensorRisingEdge = 0;	// Declare variable and initialize to 0.
char StartTimer = 0;				// Declare variable and initialize to 0.
char DisplayZero = 0;				// Declare variable and initialize to 0.
char i = 0;
char xdata display_string[9];

char Keypad_Number_1 = 1;
char Keypad_Number_4 = 0;
char Keypad_Number_7 = 0;
char Keypad_Number_star = 0;

float FeetPerSecond = 0;			// Declare variable and initialize to value 0.
float MPH = 0;						// Declare variable and initialize to value 0.
float FrequencyAverage = 0;			// Declare variable and initialize to value 0.
float TimerPeriodAverage = 0;		// Declare variable and initialize to value 0.

U8 DiscDiameter = 5;				// Declare variable and initialize to diameter (inches) of spinning disc.

U16 TimerT1 = 0;					// Declare variable and initialize to 0.
U16 TimerT2 = 0;					// Declare variable and initialize to 0.
U16 TimerPeriod = 0;				// Declare variable and initialize to 0.
U16 AverageMeasurementsSetting = 5;// Declare variable and initialize to value. Measurement counter.
U16 AverageMeasurements = 5;		// Declare variable and initialize.
U16 AverageAccumulator = 0;			// Declare variable and initialize to value 0. Used for averaging

extern U8 DisplayResults;			// Externally declared variable.
extern U16 HUNDRED_MICROSECOND_TIMER;	// Externally declared variable.

//-----------------------------------------------------------------------------
// main() Routine
// ----------------------------------------------------------------------------
int main (void)
{
	   //-------------------------------------------------
	   //Enter default mode (configure the microprocessor)
	   //-------------------------------------------------
	   enter_DefaultMode_from_RESET();

	   //----------------------------------
	   // Indicate TX0 ready.
	   //----------------------------------
	   SCON0_TI = 1;

	   //----------------------------------
	   // Basic CP240x Initialization
	   //----------------------------------
	   CP240x_Reset();
	   CP240x_RTC_Init();

	   //----------------------------------
	   // LCD Display Application Code
	   //----------------------------------
	   LCD_Init();
	   LCD_Clear();

	   //----------------------------------
	   // Main Application Loop
	   //----------------------------------

	while (1)
	{
		if((OpticalSensor == 1) && (OpticalSensorFallingEdge == 0) && (OpticalSensorRisingEdge == 0))		// If the Optical Sensor is high (blocked), and neither
																											// rising edge nor falling edge have occurred.
		{
			OpticalSensorRisingEdge = 1;	// Since the Optical Sensor signal is high (sensor is blocked) the rising edge should have occurred, so set it to 1.
		}

		else if((OpticalSensor == 0) && (OpticalSensorFallingEdge == 0) && (OpticalSensorRisingEdge == 0))	// If the Optical Sensor is low (clear), and neither
																											// rising edge nor falling edge have occurred.
		{
			OpticalSensorFallingEdge = 1;	// Since the Optical Sensor signal is low (sensor is clear) the falling edge should have occurred, so set it to 1.
		}

/*
		else if((OpticalSensor == 0) && (OpticalSensorFallingEdge == 1) && (OpticalSensorRisingEdge == 0))	// If the Optical Sensor is low (clear), and the
																											// rising edge is low and the falling edge is high,
																											// then the signal is low.
			{
				//Do nothing. The optical signal is low.
			}
*/

		else if((OpticalSensor == 1) && (OpticalSensorFallingEdge == 1) && (OpticalSensorRisingEdge == 0))	// If the Optical Sensor is high (blocked), and the
																											// rising edge is low and the falling edge is high,
																											// then the signal just transitioned from low to high.
		{
			OpticalSensorRisingEdge = 1;	// Set the rising edge (1).
			OpticalSensorFallingEdge = 0;	// Clear the falling edge (0).
		}

		else if((OpticalSensor == 1) && (OpticalSensorFallingEdge == 1) && (OpticalSensorRisingEdge == 1))	// If the Optical Sensor is high (blocked) and both the
																											// rising edge and the falling edge are high.
		{
			YELLOW_LED = 1;	// This condition shouldn't occur. Therefore, turn on the LED signifying that an error condition has occurred.
		}


		else if((OpticalSensor == 0) && (OpticalSensorFallingEdge == 0) && (OpticalSensorRisingEdge == 1))	// If the Optical Sensor is low (clear), and the
																											// falling edge is low and the rising edge is high,
																											// then the signal just transitioned from high to low.
		{
			OpticalSensorRisingEdge = 0;	// Clear the rising edge (0).
			OpticalSensorFallingEdge = 1;	// Set the falling edge (1).
			StartTimer = 1;					// Start the optical sensor period timer.
		}

		else if((OpticalSensor == 0) && (OpticalSensorFallingEdge == 1) && (OpticalSensorRisingEdge == 1))	// If the Optical Sensor is low (clear) and both the
																											// rising edge and the falling edge are high.
		{
			YELLOW_LED = 1;	// This condition shouldn't occur. Therefore, turn on the LED signifying that an error condition has occurred.
		}

/*
		else if((OpticalSensor == 1) && (OpticalSensorFallingEdge == 0) && (OpticalSensorRisingEdge == 1))	// If the Optical Sensor is high (blocked), and the
																											// rising edge is high and the falling edge is low,
																											// then the signal is high.
			{
				// Do nothing. The optical signal is high.
			}
*/

//-----------------------------------------------------------------------------
// StartTimer Routine
// ----------------------------------------------------------------------------
		if(StartTimer == 1)
		{
			TimerPeriod = 0;					// Reset
			TimerT2 = HUNDRED_MICROSECOND_TIMER;// Set TimerT2 to equal HUNDRED_MICROSECOND_TIMER value.
			TimerPeriod = (TimerT2-TimerT1);	// Take the difference between the beginning and the end of the timing cycle.
			TimerT2 = 0;						// Reset
			HUNDRED_MICROSECOND_TIMER = 0;		// Reset
			StartTimer = 0;						// Clear this flag.
			AverageAccumulator += TimerPeriod;	// Add the current timer period measurement to the accumulator.
			AverageMeasurements--;				// Decrement the measurement counter.

			if(AverageMeasurements == 0)
			{
				TimerPeriodAverage = (AverageAccumulator / 100.0);					// Calculate the average value: divide the summed AverageAccumulator by the number of measurements.
				FrequencyAverage = ((1.0/TimerPeriodAverage) * 1000.0);				// Calculate the frequency.
				FeetPerSecond = (3.14159 * DiscDiameter / 12.0 * FrequencyAverage);	// Calculation linear speed (ft/s): Pi*2*r/12*period. The "12" converts inches to feet. DiscDiameter = 5 inches.
				MPH = (0.681818 * FeetPerSecond);									// Calculation linear speed (MPH): convert ft/s to MPH.
				AverageAccumulator = 0;
				AverageMeasurements = AverageMeasurementsSetting;					// Reset
				DisplayResults = 1;													// Set flag to display results.
			}
		}

//-----------------------------------------------------------------------------
// Keypad Routine
// ----------------------------------------------------------------------------

		if(Row_1 == 1)		// P2.0
		{
			Keypad_Number_1 = 1;
			Keypad_Number_4 = 0;
			Keypad_Number_7 = 0;
			Keypad_Number_star = 0;
		}

		else if(Row_2 == 1)	// P2.1
		{
			Keypad_Number_1 = 0;
			Keypad_Number_4 = 1;
			Keypad_Number_7 = 0;
			Keypad_Number_star = 0;
		}

		else if(Row_3 == 1)	// P2.2
		{
			Keypad_Number_1 = 0;
			Keypad_Number_4 = 0;
			Keypad_Number_7 = 1;
			Keypad_Number_star = 0;
		}

		else if(Row_4 == 1)	// P2.3
		{
			Keypad_Number_1 = 0;
			Keypad_Number_4 = 0;
			Keypad_Number_7 = 0;
			Keypad_Number_star = 1;
		}

//-----------------------------------------------------------------------------
// Display Value Routine
// ----------------------------------------------------------------------------

		if(DisplayResults == 1)							// Interrupt (TIMER0_ISR and StartTimer) for when the results should be displayed.
		{
			if(AverageAccumulator < 0)					// If the time period is less than 0.
			{
				AverageAccumulator = 0;					// Set time period to 0.
				printf("\nVelocity: Calculating...");	// Note: the spaces after the units (mV) are needed to clear the terminal's line.
				printf("\033[0;0H");					// Set cursor to home position on screen.
			}

			else
			{
//				printf("\nPeriod: %3.1f ms      ", (float) TimerPeriodAverage);	// Note: the spaces after the units (mV) are needed to clear the terminal's line.
				printf("\nAngular Velocity: %3.1f rev/s          ", (float) FrequencyAverage);		// Note: the format "%3.1f" --> "3" = width of 3 positions; ".1" = 1 digit after decimal point;
				printf("\nAngular Velocity: %3.1f RPM            ", (float) (FrequencyAverage*60));	// "f" = floating point number.
				printf("\nLinear Speed (at radius of disc): %3.1f ft/s     ", (float) FeetPerSecond);
				printf("\nLinear Speed (at radius of disc): %3.1f MPH      ", (float) MPH);
				printf("\033[0;0H");																//	Set cursor to home position on screen.

				if (Keypad_Number_1 == 1)
				{
					sprintf(display_string, "%0.0f rev/s  ", (float) FrequencyAverage);
				}

				if (Keypad_Number_4 == 1)
				{
					sprintf(display_string, "%0.0f RPM  ", (float) FrequencyAverage*60);
				}

				if (Keypad_Number_7 == 1)
				{
					sprintf(display_string, "%0.0f ft/s  ", (float) FeetPerSecond);
				}

				if (Keypad_Number_star == 1)
				{
					sprintf(display_string, "%0.0f MPH  ", (float) MPH);
				}

				LCD_OutString(display_string);
			}

			// If the Optical Sensor hasn't been triggered in 5000 counts (500ms), then assume that the disc has stopped spinning and display "0".
			if(HUNDRED_MICROSECOND_TIMER >= 5000)
			{
				TimerPeriodAverage = 0;				// Reset to zero (clear time).
//				printf("\nPeriod: 0.0 ms      ");	// Note: the spaces after the units (mV) are needed to clear the terminal's line.
				printf("\nAngular Velocity: 0.0 rev/s          ");
				printf("\nAngular Velocity: 0.0 RPM            ");
				printf("\nLinear Speed (at radius of disc): 0.0 ft/s     ");
				printf("\nLinear Speed (at radius of disc): 0.0 MPH      ");
				printf("\033[0;0H");				//	Set cursor to home position on screen.

				if (Keypad_Number_1 == 1)
				{
					sprintf(display_string, "0 rev/s  ");
				}

				if (Keypad_Number_4 == 1)
				{
					sprintf(display_string, "0 RPM  ");
				}

				if (Keypad_Number_7 == 1)
				{
					sprintf(display_string, "0 ft/s  ");
				}

				if (Keypad_Number_star == 1)
				{
					sprintf(display_string, "0 MPH  ");
				}

				LCD_OutString(display_string);
			}

			DisplayResults = 0;	// Reset flag.
		}

		//	If nothing is being displayed and the optical signal hasn't been triggered for 5000 counts (500ms), then assume that the disc has stopped spinning and display "0".
		if((DisplayResults == 0) && (HUNDRED_MICROSECOND_TIMER >= 5000))
		{
			TimerPeriodAverage = 0;				// Reset to zero (clear time).
//			printf("\nPeriod: 0.0 ms      ");	// Note: the spaces after the units (mV) are needed to clear the terminal's line.
			printf("\nAngular Velocity: 0.0 rev/s          ");
			printf("\nAngular Velocity: 0.0 RPM            ");
			printf("\nLinear Speed (at radius of disc): 0.0 ft/s     ");
			printf("\nLinear Speed (at radius of disc): 0.0 MPH      ");
			printf("\033[0;0H");				//	Set cursor to home position on screen.

			if (Keypad_Number_1 == 1)
			{
				sprintf(display_string, "0 rev/s  ");
			}

			if (Keypad_Number_4 == 1)
			{
				sprintf(display_string, "0 RPM  ");
			}

			if (Keypad_Number_7 == 1)
			{
				sprintf(display_string, "0 ft/s  ");
			}

			if (Keypad_Number_star == 1)
			{
				sprintf(display_string, "0 MPH  ");
			}

			LCD_OutString(display_string);

			HUNDRED_MICROSECOND_TIMER = 0;
		}
	}	// while (1). Do forever.
}
